@@ -23,14 +23,17 @@
 #include <poppler.h>
 #include <poppler-document.h>
 #include <poppler-page.h>
+#include <glib/gi18n.h>
 
 #include "ev-poppler.h"
 #include "ev-ps-exporter.h"
 #include "ev-document-find.h"
 #include "ev-document-misc.h"
 #include "ev-document-links.h"
+#include "ev-document-fonts.h"
 #include "ev-document-security.h"
 #include "ev-document-thumbnails.h"
+#include "ev-selection.h"
 
 typedef struct {
 	PdfDocument *document;
@@ -54,6 +57,10 @@ struct _PdfDocument
 	PopplerPSFile *ps_file;
 	gchar *password;
 
+	PopplerFontInfo *font_info;
+	PopplerFontsIter *fonts_iter;
+	int fonts_scanned_pages;
+
 	PdfDocumentSearch *search;
 };
 
@@ -61,14 +68,19 @@ static void pdf_document_document_iface_init            (EvDocumentIface
 static void pdf_document_security_iface_init            (EvDocumentSecurityIface   *iface);
 static void pdf_document_document_thumbnails_iface_init (EvDocumentThumbnailsIface *iface);
 static void pdf_document_document_links_iface_init      (EvDocumentLinksIface      *iface);
+static void pdf_document_document_fonts_iface_init      (EvDocumentFontsIface      *iface);
 static void pdf_document_find_iface_init                (EvDocumentFindIface       *iface);
 static void pdf_document_ps_exporter_iface_init         (EvPSExporterIface         *iface);
+static void pdf_selection_iface_init                    (EvSelectionIface          *iface);
 static void pdf_document_thumbnails_get_dimensions      (EvDocumentThumbnails      *document_thumbnails,
 							 gint                       page,
 							 gint                       size,
 							 gint                      *width,
 							 gint                      *height);
+static int  pdf_document_get_n_pages			(EvDocument                *document);
+
 static EvLink * ev_link_from_action (PopplerAction *action);
+static void pdf_document_search_free (PdfDocumentSearch   *search);
 
 
 G_DEFINE_TYPE_WITH_CODE (PdfDocument, pdf_document, G_TYPE_OBJECT,
@@ -81,15 +93,78 @@ G_DEFINE_TYPE_WITH_CODE (PdfDocument, pdf_document, G_TYPE_OBJECT,
 							pdf_document_document_thumbnails_iface_init);
 				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_LINKS,
 							pdf_document_document_links_iface_init);
+				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_FONTS,
+							pdf_document_document_fonts_iface_init);
 				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_FIND,
 							pdf_document_find_iface_init);
 				 G_IMPLEMENT_INTERFACE (EV_TYPE_PS_EXPORTER,
 							pdf_document_ps_exporter_iface_init);
+				 G_IMPLEMENT_INTERFACE (EV_TYPE_SELECTION,
+							pdf_selection_iface_init);
 			 });
 
+
+static void
+set_rc_data (PdfDocument     *pdf_document,
+	     EvRenderContext *rc)
+{
+	if (rc->data == NULL) {
+		rc->data = poppler_document_get_page (pdf_document->document,
+						      rc->page);
+		rc->destroy = g_object_unref;
+	} else {
+		g_assert (rc->page == poppler_page_get_index (POPPLER_PAGE (rc->data)));
+	}
+}
+
+static void
+pdf_document_search_free (PdfDocumentSearch   *search)
+{
+        PdfDocument *pdf_document = search->document;
+	int n_pages;
+	int i;
+
+        if (search->idle != 0)
+                g_source_remove (search->idle);
+
+	n_pages = pdf_document_get_n_pages (EV_DOCUMENT (pdf_document));
+	for (i = 0; i < n_pages; i++) {
+		g_list_foreach (search->pages[i], (GFunc) g_free, NULL);
+		g_list_free (search->pages[i]);
+	}
+	
+        g_free (search->text);
+}
+
+static void
+pdf_document_dispose (GObject *object)
+{
+	PdfDocument *pdf_document = PDF_DOCUMENT(object);
+
+	if (pdf_document->search) {
+		pdf_document_search_free (pdf_document->search);
+		pdf_document->search = NULL;
+	}
+
+	if (pdf_document->document) {
+		g_object_unref (pdf_document->document);
+	}
+
+	if (pdf_document->font_info) { 
+		poppler_font_info_free (pdf_document->font_info);
+	}
+
+	if (pdf_document->fonts_iter) {
+		poppler_fonts_iter_free (pdf_document->fonts_iter);
+	}
+}
+
 static void
 pdf_document_class_init (PdfDocumentClass *klass)
 {
+	GObjectClass *g_object_class = G_OBJECT_CLASS (klass);
+
+	g_object_class->dispose = pdf_document_dispose;
 }
 
 static void
@@ -168,18 +243,40 @@ pdf_document_get_n_pages (EvDocument *document)
 	return poppler_document_get_n_pages (PDF_DOCUMENT (document)->document);
 }
 
+static void
+set_page_orientation (PdfDocument *pdf_document, PopplerPage *page, int rotation)
+{
+	PopplerOrientation orientation;
+
+	switch (rotation) {
+	case 90:
+		orientation = POPPLER_ORIENTATION_LANDSCAPE;
+		break;
+	case 180:
+		orientation = POPPLER_ORIENTATION_UPSIDEDOWN;
+		break;
+	case 270:
+		orientation = POPPLER_ORIENTATION_SEASCAPE;
+		break;
+	default:
+		orientation = POPPLER_ORIENTATION_PORTRAIT;
+	}
+
+	poppler_page_set_orientation (page, orientation);
+}
+
 static void
 pdf_document_get_page_size (EvDocument   *document,
 			    int           page,
 			    double       *width,
 			    double       *height)
 {
+	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerPage *poppler_page;
 
-	poppler_page = poppler_document_get_page (PDF_DOCUMENT (document)->document,
-						  page);
-
+	poppler_page = poppler_document_get_page (pdf_document->document, page);
 	poppler_page_get_size (poppler_page, width, height);
+	g_object_unref (poppler_page);
 }
 
 static char *
@@ -195,6 +292,7 @@ pdf_document_get_page_label (EvDocument *document,
 	g_object_get (G_OBJECT (poppler_page),
 		      "label", &label,
 		      NULL);
+	g_object_unref (poppler_page);
 
 	return label;
 }
@@ -233,6 +331,7 @@ pdf_document_get_links (EvDocument *document,
 	}
 
 	poppler_page_free_link_mapping (mapping_list);
+	g_object_unref (poppler_page);
 
 	return g_list_reverse (retval);
 }
@@ -240,34 +339,33 @@ pdf_document_get_links (EvDocument *document,
 
 static GdkPixbuf *
 pdf_document_render_pixbuf (EvDocument   *document,
-			    int           page,
-			    double        scale)
+			    EvRenderContext *rc)
 {
 	PdfDocument *pdf_document;
-	PopplerPage *poppler_page;
 	GdkPixbuf *pixbuf;
 	double width_points, height_points;
 	gint width, height;
 
 	pdf_document = PDF_DOCUMENT (document);
-	poppler_page = poppler_document_get_page (pdf_document->document,
-						  page);
 
-	poppler_page_get_size (poppler_page, &width_points, &height_points);
-	width = (int) ((width_points * scale) + 0.5);
-	height = (int) ((height_points * scale) + 0.5);
+	set_rc_data (pdf_document, rc);
+	set_page_orientation (pdf_document, POPPLER_PAGE (rc->data), rc->rotation);
+
+	poppler_page_get_size (POPPLER_PAGE (rc->data), &width_points, &height_points);
+	width = (int) ((width_points * rc->scale) + 0.5);
+	height = (int) ((height_points * rc->scale) + 0.5);
 
 	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
 				 FALSE, 8,
 				 width, height);
 
-	poppler_page_render_to_pixbuf (poppler_page,
+	poppler_page_render_to_pixbuf (POPPLER_PAGE (rc->data),
 				       0, 0,
 				       width, height,
-				       scale,
-				       pixbuf,
-				       0, 0);
-
+				       rc->scale,
+				       pixbuf);
+	
+	
 	return pixbuf;
 }
 
@@ -305,6 +403,7 @@ pdf_document_get_info (EvDocument *document)
 	PopplerPageLayout layout;
 	PopplerPageMode mode;
 	PopplerViewerPreferences view_prefs;
+	PopplerPermissions permissions;
 
 	info = g_new0 (EvDocumentInfo, 1);
 
@@ -315,8 +414,15 @@ pdf_document_get_info (EvDocument *document)
 			    EV_DOCUMENT_INFO_KEYWORDS |
 			    EV_DOCUMENT_INFO_LAYOUT |
 			    EV_DOCUMENT_INFO_START_MODE |
-			    /* Missing EV_DOCUMENT_INFO_CREATION_DATE | */
-			    EV_DOCUMENT_INFO_UI_HINTS;
+		            EV_DOCUMENT_INFO_PERMISSIONS |
+			    EV_DOCUMENT_INFO_UI_HINTS |
+			    EV_DOCUMENT_INFO_CREATOR |
+			    EV_DOCUMENT_INFO_PRODUCER |
+			    EV_DOCUMENT_INFO_CREATION_DATE |
+			    EV_DOCUMENT_INFO_MOD_DATE |
+			    EV_DOCUMENT_INFO_LINEARIZED |
+			    EV_DOCUMENT_INFO_N_PAGES |
+			    EV_DOCUMENT_INFO_SECURITY;
 
 
 	g_object_get (PDF_DOCUMENT (document)->document,
@@ -328,6 +434,12 @@ pdf_document_get_info (EvDocument *document)
 		      "page-mode", &mode,
 		      "page-layout", &layout,
 		      "viewer-preferences", &view_prefs,
+		      "permissions", &permissions,
+		      "creator", &(info->creator),
+		      "producer", &(info->producer),
+		      "creation-date", &(info->creation_date),
+		      "mod-date", &(info->modified_date),
+		      "linearized", &(info->linearized),
 		      NULL);
 
 	switch (layout) {
@@ -394,6 +506,30 @@ pdf_document_get_info (EvDocument *document)
 		info->ui_hints |=  EV_DOCUMENT_UI_HINT_DIRECTION_RTL;
 	}
 
+	info->permissions = 0;
+	if (permissions & POPPLER_PERMISSIONS_OK_TO_PRINT) {
+		info->permissions |= EV_DOCUMENT_PERMISSIONS_OK_TO_PRINT;
+	}
+	if (permissions & POPPLER_PERMISSIONS_OK_TO_MODIFY) {
+		info->permissions |= EV_DOCUMENT_PERMISSIONS_OK_TO_MODIFY;
+	}
+	if (permissions & POPPLER_PERMISSIONS_OK_TO_COPY) {
+		info->permissions |= EV_DOCUMENT_PERMISSIONS_OK_TO_COPY;
+	}
+	if (permissions & POPPLER_PERMISSIONS_OK_TO_ADD_NOTES) {
+		info->permissions |= EV_DOCUMENT_PERMISSIONS_OK_TO_ADD_NOTES;
+	}
+
+	info->n_pages = ev_document_get_n_pages (document);
+
+	if (ev_document_security_has_document_security (EV_DOCUMENT_SECURITY (document))) {
+		/* translators: this is the document security state */
+		info->security = g_strdup (_("Yes"));
+	} else {
+		/* translators: this is the document security state */
+		info->security = g_strdup (_("No"));
+	}
+
 	return info;
 }
 
@@ -404,6 +540,7 @@ pdf_document_get_text (EvDocument *document, int page, EvRectangle *rect)
 	PopplerPage *poppler_page;
 	PopplerRectangle r;
 	double height;
+	char *text;
 	
 	poppler_page = poppler_document_get_page (pdf_document->document, page);
 	g_return_val_if_fail (poppler_page != NULL, NULL);
@@ -414,7 +551,11 @@ pdf_document_get_text (EvDocument *document, int page, EvRectangle *rect)
 	r.x2 = rect->x2;
 	r.y2 = height - rect->y1;
 
-	return poppler_page_get_text (poppler_page, &r);
+	text = poppler_page_get_text (poppler_page, &r);
+
+	g_object_unref (poppler_page);
+
+	return text;
 }
 
 static void
@@ -439,6 +580,128 @@ pdf_document_security_iface_init (EvDocumentSecurityIface *iface)
 	iface->set_password = pdf_document_set_password;
 }
 
+static gdouble
+pdf_document_fonts_get_progress (EvDocumentFonts *document_fonts)
+{
+	PdfDocument *pdf_document = PDF_DOCUMENT (document_fonts);
+	int n_pages;
+
+        n_pages = pdf_document_get_n_pages (EV_DOCUMENT (pdf_document));
+
+	return (double)pdf_document->fonts_scanned_pages / (double)n_pages;
+}
+
+static gboolean
+pdf_document_fonts_scan (EvDocumentFonts *document_fonts,
+			 int              n_pages)
+{
+	PdfDocument *pdf_document = PDF_DOCUMENT (document_fonts);
+	gboolean result;
+
+	g_return_val_if_fail (PDF_IS_DOCUMENT (document_fonts), FALSE);
+
+	if (pdf_document->font_info == NULL) { 
+		pdf_document->font_info = poppler_font_info_new (pdf_document->document);
+	}
+
+	if (pdf_document->fonts_iter) {
+		poppler_fonts_iter_free (pdf_document->fonts_iter);
+	}
+
+	pdf_document->fonts_scanned_pages += n_pages;
+
+	result = poppler_font_info_scan (pdf_document->font_info, n_pages,
+				         &pdf_document->fonts_iter);
+	if (!result) {
+		pdf_document->fonts_scanned_pages = 0;
+		poppler_font_info_free (pdf_document->font_info);
+		pdf_document->font_info = NULL;	
+	}
+
+	return result;
+}
+
+static const char *
+font_type_to_string (PopplerFontType type)
+{
+	switch (type)
+	{
+	case POPPLER_FONT_TYPE_TYPE1:
+		return _("Type 1");
+	case POPPLER_FONT_TYPE_TYPE1C:
+		return _("Type 1C");
+	case POPPLER_FONT_TYPE_TYPE3:
+		return _("Type 3");
+	case POPPLER_FONT_TYPE_TRUETYPE:
+		return _("TrueType");
+	case POPPLER_FONT_TYPE_CID_TYPE0:
+		return _("Type 1 (CID)");
+	case POPPLER_FONT_TYPE_CID_TYPE0C:
+		return _("Type 1C (CID)");
+	case POPPLER_FONT_TYPE_CID_TYPE2:
+		return _("TrueType (CID)");
+	default:
+		return _("Unknown font type");
+	}
+}
+
+static void
+pdf_document_fonts_fill_model (EvDocumentFonts *document_fonts,
+			       GtkTreeModel    *model)
+{
+	PdfDocument *pdf_document = PDF_DOCUMENT (document_fonts);
+	PopplerFontsIter *iter = pdf_document->fonts_iter;
+
+	g_return_if_fail (PDF_IS_DOCUMENT (document_fonts));
+
+	if (!iter)
+		return;
+
+	do {
+		GtkTreeIter list_iter;
+		const char *name;
+		const char *type;
+		const char *embedded;
+		char *details;
+		
+		name = poppler_fonts_iter_get_name (iter);
+
+		if (name == NULL) {
+			name = _("No name");
+		}
+
+		type = font_type_to_string (
+			poppler_fonts_iter_get_font_type (iter));
+
+		if (poppler_fonts_iter_is_embedded (iter)) {
+			if (poppler_fonts_iter_is_subset (iter))
+				embedded = _("Embedded subset");
+			else
+				embedded = _("Embedded");
+		} else {
+			embedded = _("Not embedded");
+		}
+
+		details = g_markup_printf_escaped ("%s\n%s", type, embedded);
+
+		gtk_list_store_append (GTK_LIST_STORE (model), &list_iter);
+		gtk_list_store_set (GTK_LIST_STORE (model), &list_iter,
+				    EV_DOCUMENT_FONTS_COLUMN_NAME, name,
+				    EV_DOCUMENT_FONTS_COLUMN_DETAILS, details,
+				    -1);
+
+		g_free (details);
+	} while (poppler_fonts_iter_next (iter));
+}
+
+static void
+pdf_document_document_fonts_iface_init (EvDocumentFontsIface *iface)
+{
+	iface->fill_model = pdf_document_fonts_fill_model;
+	iface->scan = pdf_document_fonts_scan;
+	iface->get_progress = pdf_document_fonts_get_progress;
+}
+
 static gboolean
 pdf_document_links_has_document_links (EvDocumentLinks *document_links)
 {
@@ -474,7 +737,6 @@ ev_link_from_action (PopplerAction *action)
 	return link;	
 }
 
-
 static void
 build_tree (PdfDocument      *pdf_document,
 	    GtkTreeModel     *model,
@@ -487,8 +749,10 @@ build_tree (PdfDocument      *pdf_document,
 		PopplerIndexIter *child;
 		PopplerAction *action;
 		EvLink *link;
+		gboolean expand;
 		
 		action = poppler_index_iter_get_action (iter);
+		expand = poppler_index_iter_is_open (iter);
 		if (action) {
 			gtk_tree_store_append (GTK_TREE_STORE (model), &tree_iter, parent);
 			link = ev_link_from_action (action);
@@ -497,7 +761,9 @@ build_tree (PdfDocument      *pdf_document,
 			gtk_tree_store_set (GTK_TREE_STORE (model), &tree_iter,
 					    EV_DOCUMENT_LINKS_COLUMN_MARKUP, ev_link_get_title (link),
 					    EV_DOCUMENT_LINKS_COLUMN_LINK, link,
+					    EV_DOCUMENT_LINKS_COLUMN_EXPAND, expand,
 					    -1);
+			g_object_unref (link);
 			child = poppler_index_iter_get_child (iter);
 			if (child)
 				build_tree (pdf_document, model, &tree_iter, child);
@@ -517,20 +783,19 @@ pdf_document_links_get_links_model (EvDocumentLinks *document_links)
 	g_return_val_if_fail (PDF_IS_DOCUMENT (document_links), NULL);
 
 	iter = poppler_index_iter_new (pdf_document->document);
-	/* Create the model iff we have items*/
+	/* Create the model if we have items*/
 	if (iter != NULL) {
 		model = (GtkTreeModel *) gtk_tree_store_new (EV_DOCUMENT_LINKS_COLUMN_NUM_COLUMNS,
 							     G_TYPE_STRING,
-							     G_TYPE_POINTER);
+							     G_TYPE_OBJECT,
+							     G_TYPE_BOOLEAN);
 		build_tree (pdf_document, model, NULL, iter);
 		poppler_index_iter_free (iter);
 	}
 	
-
 	return model;
 }
 
-
 static void
 pdf_document_document_links_iface_init (EvDocumentLinksIface *iface)
 {
@@ -538,51 +803,57 @@ pdf_document_document_links_iface_init (EvDocumentLinksIface *iface)
 	iface->get_links_model = pdf_document_links_get_links_model;
 }
 
-
 static GdkPixbuf *
-make_thumbnail_for_size (PdfDocument *pdf_document,
-			 gint         page,
-			 gint         size,
-			 gboolean     border)
+make_thumbnail_for_size (PdfDocument   *pdf_document,
+			 gint           page,
+			 int            rotation,
+			 gint           size,
+			 gboolean       border)
 {
 	PopplerPage *poppler_page;
-	GdkPixbuf *pixbuf;
+	GdkPixbuf *pixbuf, *sub_pixbuf;
 	int width, height;
-	int x_offset, y_offset;
 	double scale;
 	gdouble unscaled_width, unscaled_height;
 
 	poppler_page = poppler_document_get_page (pdf_document->document, page);
-
+	set_page_orientation (pdf_document, poppler_page, rotation);
 	g_return_val_if_fail (poppler_page != NULL, NULL);
 
-	pdf_document_thumbnails_get_dimensions (EV_DOCUMENT_THUMBNAILS (pdf_document), page, size, &width, &height);
+	pdf_document_thumbnails_get_dimensions (EV_DOCUMENT_THUMBNAILS (pdf_document), page,
+						size, &width, &height);
 	poppler_page_get_size (poppler_page, &unscaled_width, &unscaled_height);
 	scale = width / unscaled_width;
 
 	if (border) {
 		pixbuf = ev_document_misc_get_thumbnail_frame (width, height, NULL);
-		x_offset = 1;
-		y_offset = 1;
+
+		sub_pixbuf = gdk_pixbuf_new_subpixbuf (pixbuf,
+						       1, 1,
+						       width - 1, height - 1);
 	} else {
 		pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
 					 width, height);
 		gdk_pixbuf_fill (pixbuf, 0xffffffff);
-		x_offset = 0;
-		y_offset = 0;
+		sub_pixbuf = gdk_pixbuf_new_subpixbuf (pixbuf,
+						       0, 0,
+						       width, height);
 	}
 
 	poppler_page_render_to_pixbuf (poppler_page, 0, 0,
 				       width, height,
-				       scale, pixbuf,
-				       x_offset, y_offset);
+				       scale, sub_pixbuf);
 
+	g_object_unref (G_OBJECT (sub_pixbuf));
+
+	g_object_unref (poppler_page);
 	return pixbuf;
 }
 
 static GdkPixbuf *
 pdf_document_thumbnails_get_thumbnail (EvDocumentThumbnails *document_thumbnails,
 	 			       gint 		     page,
+				       gint                  rotation,
  				       gint                  size,
 		 		       gboolean              border)
 {
@@ -593,9 +864,11 @@ pdf_document_thumbnails_get_thumbnail (EvDocumentThumbnails *document_thumbnails
 	pdf_document = PDF_DOCUMENT (document_thumbnails);
 
 	poppler_page = poppler_document_get_page (pdf_document->document, page);
+	set_page_orientation (pdf_document, poppler_page, rotation);
 	g_return_val_if_fail (poppler_page != NULL, NULL);
 
 	pixbuf = poppler_page_get_thumbnail (poppler_page);
+	
 	if (pixbuf != NULL) {
 		/* The document provides its own thumbnails. */
 		if (border) {
@@ -607,8 +880,11 @@ pdf_document_thumbnails_get_thumbnail (EvDocumentThumbnails *document_thumbnails
 		}
 	} else {
 		/* There is no provided thumbnail.  We need to make one. */
-		pixbuf = make_thumbnail_for_size (pdf_document, page, size, border);
+		pixbuf = make_thumbnail_for_size (pdf_document, page, rotation, size, border);
 	}
+
+	g_object_unref (poppler_page);
+	
 	return pixbuf;
 }
 
@@ -644,6 +920,7 @@ pdf_document_thumbnails_get_dimensions (EvDocumentThumbnails *document_thumbnail
 			*height = size;
 		}
 	}
+	g_object_unref (poppler_page);
 }
 
 static void
@@ -670,6 +947,8 @@ pdf_document_search_idle_callback (void *data)
 	matches = poppler_page_find_text (page, search->text);
 	ev_document_doc_mutex_unlock ();
 
+	g_object_unref (page);
+
 	search->pages[search->search_page] = matches;
 	ev_document_find_changed (EV_DOCUMENT_FIND (pdf_document),
 				  search->search_page);
@@ -724,25 +1003,6 @@ pdf_document_search_new (PdfDocument *pdf_document,
 	return search;
 }
 
-static void
-pdf_document_search_free (PdfDocumentSearch   *search)
-{
-        PdfDocument *pdf_document = search->document;
-	int n_pages;
-	int i;
-
-        if (search->idle != 0)
-                g_source_remove (search->idle);
-
-	n_pages = pdf_document_get_n_pages (EV_DOCUMENT (pdf_document));
-	for (i = 0; i < n_pages; i++) {
-		g_list_foreach (search->pages[i], (GFunc) g_free, NULL);
-		g_list_free (search->pages[i]);
-	}
-	
-        g_free (search->text);
-}
-
 static void
 pdf_document_find_begin (EvDocumentFind   *document,
 			 int               page,
@@ -806,7 +1066,8 @@ pdf_document_find_get_result (EvDocumentFind *document_find,
 	rectangle->y1 = height - r->y2;
 	rectangle->x2 = r->x2;
 	rectangle->y2 = height - r->y1;
-	
+	g_object_unref (poppler_page);
+		
 	return TRUE;
 }
 
@@ -869,25 +1130,30 @@ pdf_document_find_iface_init (EvDocumentFindIface *iface)
 
 static void
 pdf_document_ps_exporter_begin (EvPSExporter *exporter, const char *filename,
-				int first_page, int last_page)
+				int first_page, int last_page,
+				double width, double height, gboolean duplex)
 {
 	PdfDocument *pdf_document = PDF_DOCUMENT (exporter);
 	
 	pdf_document->ps_file = poppler_ps_file_new (pdf_document->document, filename,
 						     first_page,
 						     last_page - first_page + 1);
+	poppler_ps_file_set_paper_size (pdf_document->ps_file, width, height);
+	poppler_ps_file_set_duplex (pdf_document->ps_file, duplex);
 }
 
 static void
-pdf_document_ps_exporter_do_page (EvPSExporter *exporter, int page)
+pdf_document_ps_exporter_do_page (EvPSExporter *exporter, EvRenderContext *rc)
 {
 	PdfDocument *pdf_document = PDF_DOCUMENT (exporter);
 	PopplerPage *poppler_page;
 
 	g_return_if_fail (pdf_document->ps_file != NULL);
 
-	poppler_page = poppler_document_get_page (pdf_document->document, page);
+	poppler_page = poppler_document_get_page (pdf_document->document, rc->page);
+	set_page_orientation (pdf_document, poppler_page, rc->rotation);
 	poppler_page_render_to_ps (poppler_page, pdf_document->ps_file);
+	g_object_unref (poppler_page);
 }
 
 static void
@@ -907,6 +1173,93 @@ pdf_document_ps_exporter_iface_init (EvPSExporterIface *iface)
         iface->end = pdf_document_ps_exporter_end;
 }
 
+
+void
+pdf_selection_render_selection (EvSelection      *selection,
+				EvRenderContext  *rc,
+				GdkPixbuf       **pixbuf,
+				EvRectangle      *points,
+				EvRectangle      *old_points,
+				GdkColor        *text,
+				GdkColor        *base)
+{
+	PdfDocument *pdf_document;
+	double width_points, height_points;
+	gint width, height;
+
+	pdf_document = PDF_DOCUMENT (selection);
+	set_rc_data (pdf_document, rc);
+
+	set_page_orientation (pdf_document, POPPLER_PAGE (rc->data), rc->rotation);
+
+	poppler_page_get_size (POPPLER_PAGE (rc->data), &width_points, &height_points);
+	width = (int) ((width_points * rc->scale) + 0.5);
+	height = (int) ((height_points * rc->scale) + 0.5);
+
+	if (*pixbuf == NULL) {
+		* pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
+					   TRUE, 8,
+					   width, height);
+	}
+	
+	poppler_page_render_selection (POPPLER_PAGE (rc->data),
+				       rc->scale, *pixbuf,
+				       (PopplerRectangle *)points,
+				       (PopplerRectangle *)old_points,
+				       text,
+				       base);
+}
+
+
+GdkRegion *
+pdf_selection_get_selection_region (EvSelection     *selection,
+				    EvRenderContext *rc,
+				    EvRectangle     *points)
+{
+	PdfDocument *pdf_document;
+	GdkRegion *retval;
+
+	pdf_document = PDF_DOCUMENT (selection);
+
+	set_rc_data (pdf_document, rc);
+	set_page_orientation (pdf_document, POPPLER_PAGE (rc->data), rc->rotation);
+
+	retval = poppler_page_get_selection_region ((PopplerPage *)rc->data, rc->scale, (PopplerRectangle *) points);
+
+	return retval;
+}
+
+GdkRegion *
+pdf_selection_get_selection_map (EvSelection     *selection,
+				 EvRenderContext *rc)
+{
+	PdfDocument *pdf_document;
+	PopplerPage *poppler_page;
+	PopplerRectangle points;
+	GdkRegion *retval;
+
+	pdf_document = PDF_DOCUMENT (selection);
+	poppler_page = poppler_document_get_page (pdf_document->document,
+						  rc->page);
+	set_page_orientation (pdf_document, poppler_page, rc->rotation);
+
+	points.x1 = 0.0;
+	points.y1 = 0.0;
+	poppler_page_get_size (poppler_page, &(points.x2), &(points.y2));
+	retval = poppler_page_get_selection_region (poppler_page, 1.0, &points);
+	g_object_unref (poppler_page);
+
+	return retval;
+}
+
+static void
+pdf_selection_iface_init (EvSelectionIface *iface)
+{
+        iface->render_selection = pdf_selection_render_selection;
+        iface->get_selection_region = pdf_selection_get_selection_region;
+        iface->get_selection_map = pdf_selection_get_selection_map;
+}
+
 PdfDocument *
 pdf_document_new (void)
 {
