@@ -30,7 +30,6 @@ struct _PixbufDocument
 	GObject parent_instance;
 
 	GdkPixbuf *pixbuf;
-	EvOrientation orientation;
 };
 
 typedef struct _PixbufDocumentClass PixbufDocumentClass;
@@ -85,41 +84,6 @@ pixbuf_document_get_n_pages (EvDocument  *document)
 	return 1;
 }
 
-static EvOrientation
-pixbuf_document_get_orientation (EvDocument *document)
-{
-	PixbufDocument *pixbuf_document = PIXBUF_DOCUMENT (document);
-
-	return pixbuf_document->orientation;
-}
-
-static void
-pixbuf_document_set_orientation (EvDocument *document,
-			         EvOrientation   orientation)
-{
-	PixbufDocument *pixbuf_document = PIXBUF_DOCUMENT (document);
-
-	pixbuf_document->orientation = orientation;
-}
-
-static GdkPixbuf *
-rotate_pixbuf (EvDocument *document, GdkPixbuf *pixbuf)
-{
-	PixbufDocument *pixbuf_document = PIXBUF_DOCUMENT (document);
-
-	switch (pixbuf_document->orientation)
-	{
-		case EV_ORIENTATION_LANDSCAPE:
-			return gdk_pixbuf_rotate_simple (pixbuf, 90);
-		case EV_ORIENTATION_UPSIDEDOWN:
-			return gdk_pixbuf_rotate_simple (pixbuf, 180);
-		case EV_ORIENTATION_SEASCAPE:
-			return gdk_pixbuf_rotate_simple (pixbuf, 270);
-		default:
-			return g_object_ref (pixbuf);
-	}
-}
-
 static void
 pixbuf_document_get_page_size (EvDocument   *document,
 			       int           page,
@@ -128,29 +92,24 @@ pixbuf_document_get_page_size (EvDocument   *document,
 {
 	PixbufDocument *pixbuf_document = PIXBUF_DOCUMENT (document);
 
-	if (pixbuf_document->orientation == EV_ORIENTATION_PORTRAIT ||
-	    pixbuf_document->orientation ==  EV_ORIENTATION_UPSIDEDOWN) {
-		*width = gdk_pixbuf_get_width (pixbuf_document->pixbuf);
-		*height = gdk_pixbuf_get_height (pixbuf_document->pixbuf);
-	} else {
-		*width = gdk_pixbuf_get_height (pixbuf_document->pixbuf);
-		*height = gdk_pixbuf_get_width (pixbuf_document->pixbuf);
-	}
+	*width = gdk_pixbuf_get_width (pixbuf_document->pixbuf);
+	*height = gdk_pixbuf_get_height (pixbuf_document->pixbuf);
 }
 
 static GdkPixbuf*
-pixbuf_document_render_pixbuf (EvDocument  *document, int page, double scale)
+pixbuf_document_render_pixbuf (EvDocument      *document,
+			       EvRenderContext *rc)
 {
 	PixbufDocument *pixbuf_document = PIXBUF_DOCUMENT (document);
 	GdkPixbuf *scaled_pixbuf, *rotated_pixbuf;
 
 	scaled_pixbuf = gdk_pixbuf_scale_simple (pixbuf_document->pixbuf,
-					         gdk_pixbuf_get_width (pixbuf_document->pixbuf) * scale,
-					         gdk_pixbuf_get_height (pixbuf_document->pixbuf) * scale,
+					         gdk_pixbuf_get_width (pixbuf_document->pixbuf) * rc->scale,
+					         gdk_pixbuf_get_height (pixbuf_document->pixbuf) * rc->scale,
 					         GDK_INTERP_BILINEAR);
 
-	rotated_pixbuf = rotate_pixbuf (document, scaled_pixbuf);
-	g_object_unref (scaled_pixbuf);
+        rotated_pixbuf = gdk_pixbuf_rotate_simple (scaled_pixbuf, rc->rotation);
+        g_object_unref (scaled_pixbuf);
 
 	return rotated_pixbuf;
 }
@@ -200,18 +159,17 @@ pixbuf_document_document_iface_init (EvDocumentIface *iface)
 	iface->get_page_size = pixbuf_document_get_page_size;
 	iface->render_pixbuf = pixbuf_document_render_pixbuf;
 	iface->get_info = pixbuf_document_get_info;
-	iface->get_orientation = pixbuf_document_get_orientation;
-	iface->set_orientation = pixbuf_document_set_orientation;
 }
 
 static GdkPixbuf *
 pixbuf_document_thumbnails_get_thumbnail (EvDocumentThumbnails   *document,
 					  gint 			  page,
+					  gint 			  rotation,
 					  gint			  size,
 					  gboolean                border)
 {
 	PixbufDocument *pixbuf_document = PIXBUF_DOCUMENT (document);
-	GdkPixbuf *pixbuf;
+	GdkPixbuf *pixbuf, *rotated_pixbuf;
 	gdouble scale_factor;
 	gint height;
 	
@@ -221,8 +179,11 @@ pixbuf_document_thumbnails_get_thumbnail (EvDocumentThumbnails   *document,
 	
 	pixbuf = gdk_pixbuf_scale_simple (pixbuf_document->pixbuf, size, height,
 					  GDK_INTERP_BILINEAR);
-	
-	return pixbuf;
+
+	rotated_pixbuf = gdk_pixbuf_rotate_simple (pixbuf, rotation);
+        g_object_unref (pixbuf);
+
+        return rotated_pixbuf;
 }
 
 static void
